Module completion_format

Module completion_format 

Completion format handling Completion format handling for different shells

This module defines how completions are formatted for different shells, including support for descriptions where the shell supports them.

Enums

CompletionFormat
Represents the format in which completions should be returned