Error types and result handling Error types for the flag framework
This module defines the error types that can occur when parsing commands, flags, and arguments, or when executing command handlers.
Enums
- Error
- The main error type for the flag framework
Type Aliases
- Result
- Type alias for Results with the flag Error type